
;**************************************************************************
;*                             Alarmanlage                                *
;**************************************************************************
;*                                                                        *
;*                                                                        *
;**************************************************************************
;*          v 1.15 - 09.11.2008                                           *
;**************************************************************************
;*          Hardw.: PIC16F628A used                                       *
;*          OSC.......: Internal osc. used     POWER.....:  5V            *
;**************************************************************************

W                            EQU     H'0000'
F                            EQU     H'0001'

;----- Register Files------------------------------------------------------

INDF                         EQU     H'0000'
TMR0                         EQU     H'0001'
PCL                          EQU     H'0002'
STATUS                       EQU     H'0003'
FSR                          EQU     H'0004'
PORTA                        EQU     H'0005'
PORTB                        EQU     H'0006'

INTCON                       EQU     H'000B'
OPTION_REG                   EQU     H'0081'
TRISA                        EQU     H'0085'
TRISB                        EQU     H'0086'
CMCON                        EQU     H'001F'

;----- STATUS Bits --------------------------------------------------------
IRP                          EQU     H'0007'
RP1                          EQU     H'0006'
RP0                          EQU     H'0005'
NOT_TO                       EQU     H'0004'
NOT_PD                       EQU     H'0003'
Z                            EQU     H'0002'
DC                           EQU     H'0001'
C                            EQU     H'0000'

;==========================================================================
;
;       RAM Definition
;
;==========================================================================

    __MAXRAM H'01FF'
    __BADRAM H'07'-H'09', H'0D', H'13'-H'14', H'1B'-H'1E'
    __BADRAM H'87'-H'89', H'8D', H'8F'-H'91', H'93'-H'97', H'9E'
    __BADRAM H'105', H'107'-H'109', H'10C'-H'11F', H'150'-H'16F'
    __BADRAM H'185', H'187'-H'189', H'18C'-H'1EF'

;==========================================================================
;
;       Configuration Bits
;
;==========================================================================

_BODEN_ON                    EQU     H'3FFF'
_BODEN_OFF                   EQU     H'3FBF'
_CP_ALL                      EQU     H'03FF'
_CP_75                       EQU     H'17FF'
_CP_50                       EQU     H'2BFF'
_CP_OFF                      EQU     H'3FFF'
_DATA_CP_ON                  EQU     H'3EFF'
_DATA_CP_OFF                 EQU     H'3FFF'
_PWRTE_OFF                   EQU     H'3FFF'
_PWRTE_ON                    EQU     H'3FF7'
_WDT_ON                      EQU     H'3FFF'
_WDT_OFF                     EQU     H'3FFB'
_LVP_ON                      EQU     H'3FFF'
_LVP_OFF                     EQU     H'3F7F'
_MCLRE_ON                    EQU     H'3FFF'
_MCLRE_OFF                   EQU     H'3FDF'
_ER_OSC_CLKOUT               EQU     H'3FFF'
_ER_OSC_NOCLKOUT             EQU     H'3FFE'
_INTRC_OSC_CLKOUT            EQU     H'3FFD'
_INTRC_OSC_NOCLKOUT          EQU     H'3FFC'
_EXTCLK_OSC                  EQU     H'3FEF'
_LP_OSC                      EQU     H'3FEC'
_XT_OSC                      EQU     H'3FED'
_HS_OSC                      EQU     H'3FEE'

	__CONFIG        _BODEN_OFF & _CP_OFF & _DATA_CP_OFF & _PWRTE_ON & _WDT_OFF & _LVP_OFF & _MCLRE_ON & _INTRC_OSC_NOCLKOUT

;==========================================================================
;       Variable Definition
;==========================================================================
;INPUTS
SW1		EQU     H'00'		;SW1 is triggering RA0
SW2		EQU     H'01'		;SW2 is triggering RA1
SW3		EQU     H'02'		;SW3 is triggering RA2
SW4		EQU     H'03'		;SW4 is triggering RA3
TIMER1		EQU	H'20'		;Used in delay routine
TIMER2		EQU	H'21'		; "	"	"	

ALARM1		EQU	H'22'		;alarm status
					;bit0 = 1 => first movement detected
					;bit1 = 1 => next movement detected (within predefined time after first one)
					;bit7 = 0 => life
					;     = 1 => stand-by
ta01		EQU	H'23'
alarm_or	EQU	H'24'
keys		EQU	H'25'		;keys:

					;bit0 => no meaning

					;bit1 = 1 => key 1 was pressed and released
					;bit2 = 1 => key 2 was pressed and released
					;bit3 = 1 => key 3 was pressed and released

					;bit4 => no meaning

					;bit5 = 1 => key 3 is pressed
					;bit6 = 1 => key 2 is pressed
					;bit7 = 1 => key 1 is pressed
TIMER11		EQU	H'26'
TIMER22		EQU	H'27'
key_disp	EQU	H'28'		;displayed value during code entering
code_a		EQU	H'29'
cnt5		EQU	H'2A'
code_run	EQU	H'2B'
pirstbd		EQU	H'2C'
;==========================================================================

		ORG	0		;Reset vector address
		GOTO	RESET		;goto RESET routine when boot.

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX



;---------------------------------------------------------------------------


;	       **********************************
;              **  RESET :  main boot routine  **
;              **********************************

RESET		MOVLW	B'00000111'	;Disable Comparator module's
		MOVWF	CMCON
		;
		BSF	STATUS,RP0	;Switch to register bank 1
					;Disable pull-ups
					;INT on rising edge
					;TMR0 to CLKOUT
					;TMR0 Incr low2high trans.
					;Prescaler assign to Timer0
					;Prescaler rate is 1:256
		MOVLW	B'11010111'	;Set PIC options (See datasheet).
		MOVWF	OPTION_REG	;Write the OPTION register.
		;
		CLRF	INTCON		;Disable interrupts

		MOVLW	B'00000111'	;RB6 & RB7 - output
		MOVWF	TRISB
		MOVLW	B'11111000'	;RAx- input
		MOVWF	TRISA
		BCF	STATUS,RP0	;Switch Back to reg. Bank 0

		clrf	PORTB
;----------------------------------------------------------------------------
;PHASE 0

		clrf	keys
		clrf	code_a


start_up	bcf	PORTB,D'005'

		MOVLW   D'020'

		MOVWF   ta01

sup		clrf	PORTA
		call	init001
		call	init001
		call	init001
		call	init001

		movlw	D'007'
		movwf	PORTA
		call	init001
		call	init001
		call	init001
		call	init001

		DECFSZ  ta01,F
		GOTO    sup

		goto	main_00
;-----------------------------------------------------------------------------
init001		bsf	PORTB,D'004'
		call	dr3
		bcf	PORTB,D'004'
		call	dr3
		return
; XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
; XXXXXXXXXX       main            XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
; XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

;PHASE 1

main_00		clrf	ALARM1

		bcf	PORTB,D'006'
		bcf	PORTB,D'007'

main		movlw	D'001'
		movwf	PORTA

		btfsc	ALARM1,D'007'
		goto	stand_by		;system was deactivated

		btfsc	ALARM1,D'006'
		goto	phase9			;wrong key combination

		bcf	PORTB,D'004'		;green light
		bsf	PORTB,D'005'		;green light
		CALL	DELAY_ROUTINE
		CALL	DELAY_ROUTINE
		CALL	DELAY_ROUTINE
		CALL	DELAY_ROUTINE

		bcf	PORTB,D'005'
		CALL	DELAY_ROUTINE
		CALL	DELAY_ROUTINE
		CALL	DELAY_ROUTINE
		CALL	DELAY_ROUTINE

		btfss	ALARM1,D'000'
		goto	main
;----------------------------------------------------------------------------
;one of the sensors was active
;alarm stufe orange

		movlw	D'002'
		movwf	PORTA

;now we can wait a moment...

		call	dr4
		call	dr4
		call	dr4
		call	dr4
		call	dr4
		call	dr4
		call	dr4
		call	dr4
		call	dr4
		call	dr4
		call	dr4
		call	dr4
		call	dr4
		call	dr4
		call	dr4
		call	dr4


		call	dr4
		call	dr4
		call	dr4
		call	dr4
		call	dr4
		call	dr4
		call	dr4
		call	dr4
		call	dr4
		call	dr4
		call	dr4
		call	dr4
		call	dr4
		call	dr4
		call	dr4
		call	dr4

		MOVLW   D'050'
		MOVWF   alarm_or

;PHASE 2

main_a2		movlw	D'002'
		movwf	PORTA

		btfsc	ALARM1,D'001'
		goto	alarm_rot_p1		;rot - alarm pre-activated

		btfsc	ALARM1,D'007'
		goto	stand_by		;system was deactivated

		btfsc	ALARM1,D'006'
		goto	phase9			;wrong key combination

		bcf	PORTB,D'004'		;green light
		bsf	PORTB,D'005'		;green light
		CALL	dr2
		CALL	dr2
		CALL	dr2
		CALL	dr2

		bsf	PORTB,D'004'		;red light
		bcf	PORTB,D'005'		;red light
		CALL	dr2
		CALL	dr2
		CALL	dr2
		CALL	dr2

                DECFSZ  alarm_or,F
                GOTO    main_a2

;nothing happens in the meantime
;return to standard mode

		goto	main_00
;--------------------------------------------------------------------------------

;alarm stufe rot
;until now it is only possible to switch off the alarm with a keyboard code

;PHASE 3

alarm_rot_p1	MOVLW   D'060'			; ~2 min
		MOVWF   ta01

alarm1		movlw	D'003'
		movwf	PORTA

		bsf	PORTB,D'004'		;red light
		bcf	PORTB,D'005'		;red light

		btfsc	ALARM1,D'007'
		goto	stand_by		;system was deactivated

		btfsc	ALARM1,D'006'
		goto	phase9			;wrong key combination

		CALL	dr2
		CALL	dr2
		CALL	dr2
		CALL	dr2
		CALL	dr2

		bcf	PORTB,D'004'		;light off
		movlw	D'007'
		movwf	PORTA		

		CALL	dr2
		CALL	dr2
		CALL	dr2
		CALL	dr2
		CALL	dr2

                DECFSZ  ta01,F
                GOTO    alarm1

;----------------------------------------------------------------------------
;alarm stufe ROT
;switch on the SIREN

;PHASE 4

alarm2		bsf	PORTB,D'006'
		bsf	PORTB,D'007'

		bsf	PORTB,D'003'

		bsf	PORTB,D'004'		;red light
		bcf	PORTB,D'005'		;red light

		movlw	D'005'
		movwf	PORTA

;PHASE 4/1 - siren on ---------------------------------------------------

		MOVLW   D'010'			;how long? (phase 4/1)
		MOVWF   ta01

end_401		call	dr4
		call	dr4
		bcf	PORTB,D'004'		;light off
		call	dr4
		call	dr4
		bsf	PORTB,D'004'		;red light
		call	dr4

		btfsc	ALARM1,D'007'
		goto	stand_by		;system was deactivated

		call	dr4
		bcf	PORTB,D'004'		;light off
		call	dr4
		call	dr4
		bsf	PORTB,D'004'		;red light
		call	dr4
		call	dr4

                DECFSZ  ta01,F
                GOTO    end_401


;PHASE 4/2 - silence --------------------------------------------------

		bcf	PORTB,D'006'
		bcf	PORTB,D'007'

		MOVLW   D'005'			;how long? (phase 4/2)
		MOVWF   ta01

end_402		call	dr4
		call	dr4
		bcf	PORTB,D'004'		;light off
		call	dr4
		call	dr4
		bsf	PORTB,D'004'		;red light
		call	dr4

		btfsc	ALARM1,D'007'
		goto	stand_by		;system was deactivated

		call	dr4
		bcf	PORTB,D'004'		;light off
		call	dr4
		call	dr4
		bsf	PORTB,D'004'		;red light
		call	dr4
		call	dr4

                DECFSZ  ta01,F
                GOTO    end_402

;PHASE 4/3 - siren on ---------------------------------------------------

		bsf	PORTB,D'006'
		bsf	PORTB,D'007'

		MOVLW   D'020'			;how long? (phase 4/3)
		MOVWF   ta01

end_403		call	dr4
		call	dr4
		bcf	PORTB,D'004'		;light off
		call	dr4
		call	dr4
		bsf	PORTB,D'004'		;red light
		call	dr4

		btfsc	ALARM1,D'007'
		goto	stand_by		;system was deactivated

		call	dr4
		bcf	PORTB,D'004'		;light off
		call	dr4
		call	dr4
		bsf	PORTB,D'004'		;red light
		call	dr4
		call	dr4

                DECFSZ  ta01,F
                GOTO    end_403

		goto	main_00
;----------------------------------------------------------------------------
;wrong key combination

phase9		movlw	D'050'
		movwf	ta01

p9l2		movlw	D'005'
		movwf	PORTA
		call	dr9
		movlw	D'007'
		movwf	PORTA
		call	dr9

                DECFSZ  ta01,F
                GOTO    p9l2

		goto	alarm2
;----------------------------------------------------------------------------
alarm_x		bsf	ALARM1,D'000'
		return

alarm_x2	bsf	ALARM1,D'001'
		return
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
stand_by	bcf	PORTB,D'006'		;switch off siren 1
		bcf	PORTB,D'007'		;switch off siren 2

		movlw	D'007'
		movwf	PORTA			;switch off the display

		bcf	PORTB,D'004'		;green light
		bsf	PORTB,D'005'		;green light

wait_sb		call	dr6

		btfss	ALARM1,D'007'
		goto	wake_up			;system was activated

		goto	wait_sb
;---------------------------------------------------------------------------
wake_up		MOVLW   D'120'			; ~4 min
		MOVWF   ta01

wup1		movlw	D'004'
		movwf	PORTA

		bcf	PORTB,D'004'		;green light
		bsf	PORTB,D'005'		;green light

		CALL	dr5
		CALL	dr5
		CALL	dr5
		CALL	dr5
		CALL	dr5

		bcf	PORTB,D'005'		;light off
		movlw	D'007'
		movwf	PORTA		

		CALL	dr5
		CALL	dr5
		CALL	dr5
		CALL	dr5
		CALL	dr5

                DECFSZ  ta01,F
                GOTO    wup1

		goto	main_00
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;used in phase1 (quite / no movement)

DELAY_ROUTINE	MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOP1       MOVLW   D'255'
		MOVWF   TIMER1


		call	check_pir
		call	key_smoke


DEL_LOOP2       DECFSZ  TIMER1,F
                GOTO    DEL_LOOP2
                DECFSZ  TIMER2,F
                GOTO    DEL_LOOP1
		RETLW   0
;----------------------------------------------------------------------------
;used in phase2 & phase3 (after first / next movement)

dr2		MOVLW   D'255'
		MOVWF   TIMER2
DL2_1		MOVLW   D'255'
		MOVWF   TIMER1


		call	check_pir2
		call	key_smoke


DL2_2		DECFSZ  TIMER1,F
                GOTO    DL2_2
                DECFSZ  TIMER2,F
                GOTO    DL2_1
		return
;---------------------------------------------------------------------------
;used within init phase (phase0)

dr3		MOVLW   D'150'
		MOVWF   TIMER2
DL3_1		MOVLW   D'255'
		MOVWF   TIMER1

DL3_2		DECFSZ  TIMER1,F
                GOTO    DL3_2
                DECFSZ  TIMER2,F
                GOTO    DL3_1
		return
;---------------------------------------------------------------------------
;used in Phase4 (siren on)

dr4		MOVLW   D'255'
		MOVWF   TIMER2
DL4_1		MOVLW   D'255'
		MOVWF   TIMER1

		call	key_smoke

DL4_2		DECFSZ  TIMER1,F
                GOTO    DL4_2
                DECFSZ  TIMER2,F
                GOTO    DL4_1
		return
;---------------------------------------------------------------------------
;used during wake-up

dr5		MOVLW   D'255'
		MOVWF   TIMER2
DL5_1		MOVLW   D'255'
		MOVWF   TIMER1

DL5_2		DECFSZ  TIMER1,F
                GOTO    DL5_2
                DECFSZ  TIMER2,F
                GOTO    DL5_1
		return

;---------------------------------------------------------------------------
;used in stand by mode

dr6		MOVLW   D'255'
		MOVWF   TIMER2
DL6_1		MOVLW   D'255'
		MOVWF   TIMER1

;		call	chk_pir_stb		;not used

		btfss	PORTA,D'003'
		goto	DL6_2
		btfss	PORTA,D'004'
		goto	DL6_2
		btfsc	PORTA,D'006'
		call	activate

DL6_2		DECFSZ  TIMER1,F
                GOTO    DL6_2
                DECFSZ  TIMER2,F
                GOTO    DL6_1
		return
;--------------------------------------------------------------------------
dr7		MOVLW   D'150'
		MOVWF   TIMER22
DL7_1		MOVLW   D'255'
		MOVWF   TIMER11

DL7_2		DECFSZ  TIMER11,F
                GOTO    DL7_2
                DECFSZ  TIMER22,F
                GOTO    DL7_1
		return
;-------------------------------------------------------------
;used in phase9
dr9		MOVLW   D'255'
		MOVWF   TIMER2
DL9_1		MOVLW   D'255'
		MOVWF   TIMER1

DL9_2		DECFSZ  TIMER1,F
                GOTO    DL9_2
                DECFSZ  TIMER2,F
                GOTO    DL9_1
		return

;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;deactivation using "keyboard"
keyboard	btfsc	PORTA,D'003'
		call	bt01_on
		btfsc	PORTA,D'006'
		call	bt02_on
		btfsc	PORTA,D'004'
		call	bt03_on

		btfss	PORTA,D'003'
		call	bt01_off
		btfss	PORTA,D'006'
		call	bt02_off
		btfss	PORTA,D'004'
		call	bt03_off

		return
;----------------------------------------------------------------------------
bt01_on		bcf	keys,D'001'
		bsf	keys,D'007'
		return

bt02_on		bcf	keys,D'002'
		bsf	keys,D'006'
		return

bt03_on		bcf	keys,D'003'
		bsf	keys,D'005'
		return

bt01_off	btfsc	keys,D'007'	;not pressed before
		bsf	keys,D'001'
		return

bt02_off	btfsc	keys,D'006'	;not pressed before
		bsf	keys,D'002'
		return

bt03_off	btfsc	keys,D'005'	;not pressed before
		bsf	keys,D'003'
		return
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
check_pir	btfsc	PORTB,D'000'
		call	alarm_x
		btfsc	PORTB,D'001'
		call	alarm_x
		btfsc	PORTB,D'002'
		call	alarm_x

		return

;---------------------------------------------------------------------------
chk_pir_stb	clrf	pirstbd

		btfsc	PORTB,D'000'
		call	pirstb_1
		btfsc	PORTB,D'001'
		call	pirstb_2
		btfsc	PORTB,D'002'
		call	pirstb_3

		btfsc	pirstbd,D'000'
		return

		movlw	D'007'
		movwf	PORTA

		return
;-----------------------------------
pirstb_1	movlw	D'001'
		movwf	PORTA
		bsf	pirstbd,D'000'
		return

pirstb_2	movlw	D'002'
		movwf	PORTA
		bsf	pirstbd,D'000'
		return

pirstb_3	movlw	D'003'
		movwf	PORTA
		bsf	pirstbd,D'000'
		return
;------------------------------------------------------------------------------
check_pir2	btfsc	PORTB,D'000'
		call	alarm_x2
		btfsc	PORTB,D'001'
		call	alarm_x2
		btfsc	PORTB,D'002'
		call	alarm_x2

		return

;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
activate	bcf	ALARM1,D'007'
		return

deactivate	bsf	ALARM1,D'007'		;system deactivated
		clrf	keys
		return
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;             KEY SMOKE
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

key_smoke	clrf	code_run

		movlw	D'050'
		movwf	cnt5

ks_loop		call	keyboard

		btfsc	keys,D'001'
		call	disp_1

		btfsc	keys,D'002'
		call	disp_2

		btfsc	keys,D'003'
		call	disp_3

		btfsc	code_a,D'000'
		goto	ks_cont
		return			;nothing pressed

ks_cont		call	disp_value

;---------------------------------------
;analyze the code
		btfss	code_run,D'003'
		goto	chk_frs
		btfss	code_run,D'002'
		goto	chk_sec
		btfss	code_run,D'001'
		goto	chk_thr
		btfss	code_run,D'000'
		goto	chk_for

		goto	ks_cont2
;---------------------------------------
;---------------------------------------
chk_frs		btfss	key_disp,D'003'		;first number of the code
		goto	key_wrong
		bsf	code_run,D'003'		;first number of the code was set correctly
		goto	ks_cont2

chk_sec		btfsc	key_disp,D'003'		;character before
		goto	ks_cont2
		btfss	key_disp,D'001'		;second number of the code
		goto	key_wrong
		bsf	code_run,D'002'		;second number of the code was correct
		goto	ks_cont2

chk_thr		btfsc	key_disp,D'001'		;char before
		goto	ks_cont2
		btfss	key_disp,D'002'		;third number of the code
		goto	key_wrong
		bsf	code_run,D'001'		;third number of the code was correct
		goto	ks_cont2

chk_for		btfsc	key_disp,D'002'		;char before
		goto	ks_cont2
		btfss	key_disp,D'001'		;fourth number of the code
		goto	key_wrong
		bsf	code_run,D'000'		;fourth number of the code was correct

;whole code was entered corretly
;now we have to deactivate the station

		bsf	ALARM1,D'007'
		goto	ks_exit
;---------------------------------------

ks_cont2	call	dr7
		movlw	D'007'
		movwf	PORTA
		call	dr7
                DECFSZ  cnt5,F
		goto	ks_loop

ks_exit		clrf	keys
		clrf	key_disp
		clrf	code_a
		return
;----------------------------------------
;wrong key was pressed
key_wrong	bsf	ALARM1,D'006'		;switch on siren (later)

		movlw	D'005'			;display '9'
		movwf	PORTA

		goto	ks_exit
;------------------------------------------------------------------------
disp_1		movlw	B'00000010'
		movwf	key_disp
		clrf	keys
		bsf	code_a,D'000'
		return

disp_2		movlw	B'00000100'
		movwf	key_disp
		clrf	keys
		bsf	code_a,D'000'
		return

disp_3		movlw	B'00001000'
		movwf	key_disp
		clrf	keys
		bsf	code_a,D'000'
		return
;-------------------------------------------------------------------------
disp_value	movlw	D'000'

		btfsc	key_disp,D'001'
		movlw	D'001'
		btfsc	key_disp,D'002'
		movlw	D'002'
		btfsc	key_disp,D'003'
		movlw	D'003'

		movwf	PORTA
		return
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

		END